
package Bio::Pipeline::Comparison;
# ABSTRACT: Comparative assesment of variant calling (CAVar)
use Moose;

no Moose;
__PACKAGE__->meta->make_immutable;
1;

__END__

=pod

=head1 NAME

Bio::Pipeline::Comparison - Comparative assesment of variant calling (CAVar)

=head1 VERSION

version 1.123050

=head1 AUTHOR

Andrew J. Page <ap13@sanger.ac.uk>

=head1 COPYRIGHT AND LICENSE

This software is Copyright (c) 2012 by Wellcome Trust Sanger Institute.

This is free software, licensed under:

  The GNU General Public License, Version 3, June 2007

=cut
